@@ -57,6 +57,7 @@ import javax.media.j3d.Texture;
 import javax.media.j3d.Transform3D;
 import javax.media.j3d.TransformGroup;
 import javax.media.j3d.TransparencyAttributes;
+import javax.vecmath.AxisAngle4f;
 import javax.vecmath.Color3f;
 import javax.vecmath.Point3d;
 import javax.vecmath.Vector3d;
@@ -84,6 +85,7 @@ import com.sun.j3d.utils.image.TextureLoader;
  * <a href="http://www.khronos.org/files/collada_spec_1_4.pdf">http://www.khronos.org/files/collada_spec_1_4.pdf</a>.
  * All texture coordinates are considered to belong to the same set (for example UVSET0).
  * @author Emmanuel Puybaret
+ * @author apptaro (bug fixes)
  */
 public class DAELoader extends LoaderBase implements Loader {
   /**
@@ -188,6 +190,8 @@ public class DAELoader extends LoaderBase implements Loader {
     private final Map<String, float []> positions = new HashMap<String, float []>();
     private final Map<String, float []> normals = new HashMap<String, float []>();
     private final Map<String, float []> textureCoordinates = new HashMap<String, float []>();
+    private final Map<String, float []> floatArrays = new HashMap<String, float []>();
+    private final Map<float [], Integer> sourceAccessorStrides = new HashMap<float [], Integer>();
     private final Map<Geometry, String> geometryAppearances = new HashMap<Geometry, String>();
     private final List<int []> facesAndLinesPrimitives = new ArrayList<int[]>();
     private final List<int []> polygonsPrimitives = new ArrayList<int[]>();
@@ -217,6 +221,7 @@ public class DAELoader extends LoaderBase implements Loader {
     private String  geometryId;
     private String  meshSourceId;
     private String  verticesId;
+    private String  floatArrayId;
     private String  geometryAppearance;
     private int     geometryVertexOffset;
     private int     geometryNormalOffset;
@@ -297,6 +302,15 @@ public class DAELoader extends LoaderBase implements Loader {
           this.meshSourceId = attributes.getValue("id");
         } else if ("mesh".equals(parent) && "vertices".equals(name)) {
           this.verticesId = attributes.getValue("id");
+        } else if (this.meshSourceId != null) {
+          if ("float_array".equals(name)) {
+            this.floatArrayId = attributes.getValue("id");
+          } else if ("technique_common".equals(parent) && "accessor".equals(name)) {
+            String floatArrayAnchor = attributes.getValue("source").substring(1);
+            String stride = attributes.getValue("stride");
+            this.sourceAccessorStrides.put(this.floatArrays.get(floatArrayAnchor), 
+                stride != null ? Integer.valueOf(stride) : 1);
+          } 
         } else if (this.verticesId != null && "input".equals(name)) {
           String sourceAnchor = attributes.getValue("source").substring(1);
           if ("POSITION".equals(attributes.getValue("semantic"))) {
@@ -307,9 +321,11 @@ public class DAELoader extends LoaderBase implements Loader {
             this.textureCoordinates.put(this.verticesId, this.sources.get(sourceAnchor));
           }
         } else if (this.verticesId == null && "input".equals(name)) {
-          this.inputCount++;
           String sourceAnchor = attributes.getValue("source").substring(1);
           int offset = Integer.parseInt(attributes.getValue("offset"));
+          if (this.inputCount < offset + 1) {
+            this.inputCount = offset + 1;
+          }
           if ("VERTEX".equals(attributes.getValue("semantic"))) {
             this.geometryVertices = this.positions.get(sourceAnchor);
             this.geometryVertexOffset = offset;
@@ -393,7 +409,7 @@ public class DAELoader extends LoaderBase implements Loader {
               }
             });
         }
-      } else if ("instance_material".equals(name)) {
+      } else if ("instance_material".equals(name) && !this.parentGroups.empty()) {
         String materialInstanceTarget = attributes.getValue("target");
         if (materialInstanceTarget.startsWith("#")) {
           final String materialInstanceAnchor = materialInstanceTarget.substring(1);
@@ -459,7 +475,14 @@ public class DAELoader extends LoaderBase implements Loader {
         int floatCount = 0;
         for (int i = 0; i < floatValues.length; i++) {
           if (floatValues [i].length() > 0) {
-            this.floats [floatCount++] = Float.parseFloat(floatValues [i]);
+            float floatValue;
+            try {
+              floatValue = Float.parseFloat(floatValues [i]);
+            } catch (NumberFormatException e) {
+              // This may happen with some bad DAE files
+              floatValue = 0f;
+            }
+            this.floats [floatCount++] = floatValue;
           }
         }
         if (floatCount != floatValues.length) {
@@ -467,6 +490,10 @@ public class DAELoader extends LoaderBase implements Loader {
           System.arraycopy(this.floats, 0, floats, 0, floatCount);
           this.floats = floats;        
         }
+        if (this.floatArrayId != null) {
+          this.floatArrays.put(this.floatArrayId, this.floats);
+          this.floatArrayId = null;
+        }
       } else if ("float".equals(name)) {
         this.floatValue = Float.parseFloat(getCharacters());
       }
@@ -495,13 +522,7 @@ public class DAELoader extends LoaderBase implements Loader {
         mulTransformGroup(new Transform3D(this.floats));
       } else if ("node".equals(parent) && "rotate".equals(name)) {
         Transform3D rotation = new Transform3D();
-        if (floats [0] > 0) {
-          rotation.rotX(Math.toRadians(floats [3]));
-        } else if (floats [1] > 0) {
-          rotation.rotY(Math.toRadians(floats [3]));
-        } else if (floats [2] > 0) {
-          rotation.rotZ(Math.toRadians(floats [3]));
-        }
+        rotation.setRotation(new AxisAngle4f(floats[0], floats[1], floats[2], (float)Math.toRadians(floats[3])));
         mulTransformGroup(rotation);
       } else if ("scale".equals(name)) {
         Transform3D scale = new Transform3D();
@@ -700,6 +721,7 @@ public class DAELoader extends LoaderBase implements Loader {
     private void handleGeometryElementsEnd(String name, String parent) {
       if ("mesh".equals(parent) && "source".equals(name)) {
         this.sources.put(this.meshSourceId, this.floats);
+        this.meshSourceId = null;
       } else if ("mesh".equals(parent) && "vertices".equals(name)) {
         this.verticesId = null;
       } else if ("p".equals(name)
@@ -786,8 +808,20 @@ public class DAELoader extends LoaderBase implements Loader {
         geometryInfo.setNormalIndices(getIndices(this.geometryNormalOffset));
       }
       if (this.geometryTextureCoordinates != null) {
+        Integer stride = this.sourceAccessorStrides.get(this.geometryTextureCoordinates);
+        // Support only UV texture coordinates
+        float [] textureCoordinates;
+        if (stride > 2) {
+          textureCoordinates = new float [this.geometryTextureCoordinates.length / stride * 2];
+          for (int i = 0, j = 0; j < this.geometryTextureCoordinates.length; j += stride) {
+            textureCoordinates [i++] = this.geometryTextureCoordinates [j];
+            textureCoordinates [i++] = this.geometryTextureCoordinates [j + 1];
+          }
+        } else {
+          textureCoordinates = this.geometryTextureCoordinates;
+        }
         geometryInfo.setTextureCoordinateParams(1, 2);
-        geometryInfo.setTextureCoordinates(0, this.geometryTextureCoordinates);
+        geometryInfo.setTextureCoordinates(0, textureCoordinates);
         geometryInfo.setTextureCoordinateIndices(0, getIndices(this.geometryTextureCoordinatesOffset));
       }
       
@@ -944,7 +978,7 @@ public class DAELoader extends LoaderBase implements Loader {
           axisTransform.rotX(-Math.PI / 2);
         } else if ("X_UP".equals(axis)) {
           axisTransform.rotZ(Math.PI / 2);
-        } 
+        }
         rootTransform.mul(axisTransform);
         // Translate model to its center
         Point3d lower = new Point3d();
